module Sluggable
  extend ActiveSupport::Concern

  included do
    before_save :generate_slug!
  end

  def to_param
    self.slug
  end

  def generate_slug!
    the_slug = to_slug(value_to_slug)
    obj = self.class.find_by(slug: the_slug)
    count = 2
    while obj && obj != self
      the_slug = append_suffix(the_slug, count)
      obj = self.class.find_by(slug: the_slug)
      count += 1
    end
    self.slug = the_slug.downcase
  end

  def value_to_slug
    fail NotImplementedError, "Abstract method: defined by targeted subclass."
  end

  def to_slug(name)
    str = name.strip
    str.gsub!(/\s*[^A-Za-z0-9]\s*/, "-")
    str.gsub!(/-+/, "-")
    str.downcase
  end

  def append_suffix(str, count)
    if str.split("-").last.to_i != 0
      return str.split("-").slice(0...-1).join("-") + "-" + count.to_s
    else
      return str + "-" + count.to_s
    end
  end
end